//--*-coding:utf-8-*--

package {
    
    import com.adobe.serialization.json.JSON;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.ProgressEvent;
    import flash.net.FileFilter;
    import flash.net.FileReference;
    import flash.net.sendToURL;
    import flash.net.URLLoader;
    import flash.net.URLLoaderDataFormat;
    import flash.net.URLRequest;
    import flash.net.URLRequestMethod;
    import flash.net.URLVariables;
    import flash.utils.*;
    import mx.controls.Alert;
    import com.nextgenapp.wave.gadget.*;
    import mx.utils.StringUtil;
    
    public class SharedDataSenderForGoogleWave extends SharedDataSender {
        
        private var thisObj:SharedDataSenderForGoogleWave = null;
        
        private var wave:Wave = null;
        
        public function SharedDataSenderForGoogleWave() {
            thisObj = this;
            receiver.setSender(this);
            initWave();
        }
        
        private function initWave():void {
            wave = new Wave();
            initDefaultSaveData();
        }
        
        override public function checkRoomStatus(roomNumber:int, adminPassword:String, resultFunction:Function):void {
            var jsonData:Object = {
                "isRoomExist" : true,
                "roomName" : "GoogleWave",
                "roomNumber" : roomNumber,
                "isPasswordLocked" : false,
                "isMentenanceModeOn" : false
            }
            resultFunction(jsonData);
        }
        
        override public function loginPassword(roomNumber:int, password:String, resultFunction:Function):void {
        }
        
        private function getSaveTargetTypes(saveFileName:String):Array {
            var regExp:RegExp = new RegExp("\." + mapSaveFileExtension + "$");
            var regResult:Object = regExp.exec(saveFileName);
            if( regResult != null ) {
                return ['mapData', 'characters', 'graveyard'];
            }
            
            var types:Array = new Array();
            for(var key:String in defaultSaveData) {
                types.push(key);
            }
            return types;
        }
        
        private function getSaveText(types:Array):String {
            var saveDataAll:Object = new Object();
            
            for(var i:int = 0 ; i < types.length ; i++) {
                var key:String = types[i];
                var data:Object = getSaveData(key);
                saveDataAll[key] = data;
            }
            
            return Utils.getJsonString( saveDataAll );
        }
        
        override public function save(resultFunction:Function):void {
            var saveFileName:String = getNewSaveFileName(saveFileExtension);
            var jsonData:Object = {"saveFileName" : saveFileName};
            resultFunction(jsonData);
        }
        
        override public function saveMap(resultFunction:Function):void {
            var saveFileName:String = getNewSaveFileName(mapSaveFileExtension);
            var jsonData:Object = {"saveFileName" : saveFileName};
            resultFunction(jsonData);
        }
        
        private function getNewSaveFileName(extension:String):String {
            var date:Date = new Date();
            return StringUtil.substitute("DodontoF_{0}_{1}{2}_{3}{4}{5}.{6}", 
                                         date.fullYear,
                                         date.month,
                                         date.date,
                                         date.hours,
                                         date.minutes,
                                         date.seconds,
                                         extension);
        }
        
        
        override protected function getFileSelectHandlerForLoad(resultFunction:Function):Function {
            return function(event:Event):void {
                var fileReference:FileReference = event.currentTarget as FileReference;
                if( fileReference == null ) {
                    return;
                }
                
                fileReference.addEventListener(Event.COMPLETE, fileCompleateHandlerForLoad);
                fileReference.load();
                
                if( resultFunction != null ) {
                    resultFunction();
                }
            }
        }
        
        private function fileCompleateHandlerForLoad(event:Event):void {
            var fileReference:FileReference = event.currentTarget as FileReference;
            
            var data:ByteArray = fileReference.data as ByteArray;
            loadToSaveData(data);
        }
        
        public function loadToSaveData(data:ByteArray):void {
            var dataString:String = data.toString();
            var saveData:Object = Utils.getJsonDataFromString(dataString);
            
            for(var key:String in saveData) {
                setSaveData(key, saveData[key]);
            }
        }
        
        override public function uploadImageData(params:Object, resultFunction:Function, errorFunction:Function):void {
        }
        
        override public function resurrectCharacter(resurrectCharacterId:String):void {
            var characters:Array = getSaveData("characters") as Array;
            var graveyard:Array = getSaveData("graveyard") as Array;
            
            var index:int = -1;
            var characterData:Object = null;
            for(var i:int = 0 ; i < graveyard.length ; i++) {
                characterData = graveyard[i];
                if( characterData['imgId'] == resurrectCharacterId ) {
                    index = i;
                }
            }
            
            if( index == -1 ) {
                return;
            }
            
            graveyard.splice(index, 1); 
            characters.push(characterData);
            
            setSaveData("characters", characters);
            setSaveData("graveyard", graveyard);
        }
        
        override public function sendRoundTimeData(round:int, initiative:Number, counterNames:Array):void {
            var roundTimeData:Object = {"round": round,
                                        "initiative": initiative,
                                        "counterNames": counterNames};
            setSaveData("roundTimeData", roundTimeData);
        }
        
        
        override public function startRefresh():void {
            refresh();
            wave.setStateCallback( refresh );
        }
        
        override public function clearRetry():Boolean {
            return false;
        }
        
        override public function logout():void {
            stopRefresh();
        }
        
        
        /////////////////////////////////////////
        
        
        private function getSaveDataString(type:String):String {
            
            var state:WaveState = wave.getState();
            return state.getStringValue( type );
        }
        
        private function getSaveData(type:String):Object {
            Log.logging("getSaveData begin. type", type);
            
            var saveDataString:String = getSaveDataString(type);
            
            if( saveDataString != null ) {
                Log.logging(type, saveDataString);
                return Utils.getJsonDataFromString(saveDataString);
            }
            
            Log.logging(type + " is NULL so get default" );
            
            var saveData:Object = defaultSaveData[type];
            if( saveData == null ) {
                Alert.show("type : " + type + " is unknown saveData type.");
            }
            
            Log.logging("getSaveData end.");
            
            return getClone(saveData);
        }
        
        private function setSaveDataList(dataList:Array):void {
            for(var i:int = 0 ; i < dataList.length ; i++) {
                var data:Object = dataList[i];
                var type:String = data[0];
                var obj:Object = data[1];
                setSaveData(type, obj, false);
            }
        }
        
        private function setSaveData(type:String, obj:Object, isRefresh:Boolean = true):void {
            var objString:String = Utils.getJsonString(obj);
            Log.logging("setSaveData objString", objString);
            
            
            Log.logging("setSaveData pass1");
            var delta:Object = new Object();
            delta[type] = objString;
            
            Log.logging("setSaveData pass2");
            var state:WaveState = wave.getState();
            state.submitDelta( delta );
            Log.logging("setSaveData pass3");
        }
        
        /*
          private function testWave():void {
          Alert.show("in wave container=" + wave.isInWaveContainer() + ".  time=" + wave.getTime());
          var viewer:WaveParticipant = wave.getViewer();
          if( viewer != null ) {
          Alert.show("viewer: id=" + viewer.id + " displayName=" + viewer.displayName + ".   thumbnailUrl=" + viewer.thumbnailUrl);
          }
          }
        */
        /////////////////////////////////////////
        
        private var defaultSaveData:Object = new Object();
        
        private function initDefaultSaveData():void {
            defaultSaveData = {
                "lastUpdateTimes" : this.lastUpdateTimes,
                "refreshIndex" : this.refreshIndex,
                "loginUserInfo" : [],
                
                "characters" :
                [{"name":"sample","imgId":"character_1260575421.171_1","size":1,"x":8,"y":4,"draggable":true,"isHide":false,"info":"","type":"characterData","imageName":"image\/defaultImageSet\/chara01_a5_24_2.png","direction":0,"rotation":0,"initiative":0}],
                
                "graveyard" :
                [],
                
                "roundTimeData" :
                {"counterNames":["HP","MP"],"round":1,"initiative":0},
                
                "chatMessageDataLog" :
                [[0,{"senderName":"どどんとふ","uniqueId":"1260575380.062","messageIndex":-1,"color":"00AA00","message":"「ななしさん」がログインしました。"}]],
                
                "effects":
                [{"effectId":"1228488734.998","mx_internal_uid":"EF58BC99-5BC7-2809-9D6A-113F92FF3F81","displaySeconds":0.0,"message":"驚愕","source":".\/movie\/shout.flv"},{"effectId":"1228623756.228","mx_internal_uid":"8DE523DC-0945-543A-0167-01934FCDC887","height":0.0,"displaySeconds":4.0,"message":"眼鏡さん","source":".\/image\/stand\/glass_smile.png","width":0.0},{"name":"スポーツ娘","effectId":"1228825137.001","type":"standingGraphicInfos","source":"image\/stand\/sports_normal.png","state":"通常"},{"name":"スポーツ娘","effectId":"1228825151.579","mx_internal_uid":"61763FB4-46A9-4EE4-589B-212E023F5699","type":"standingGraphicInfos","source":"image\/stand\/sports_angry.png","state":"怒り"}],
                
                "mapData" :
                {"yMax":20,"mapMarks":[],"imageSource":"image\/defaultImageSet\/\/feeld001.gif","mapType":"imageGraphic","xMax":20,"gridColor":0}
            }
        }
        
        override public function refresh(obj:Object = null):void {
            if( isStopRefreshOn ) {
                return;
            }
            
            var jsonData:Object = new Object();
            var types:Array = ["characters", "graveyard", "roundTimeData", "chatMessageDataLog", "effects",
                               "mapData", "lastUpdateTimes", "refreshIndex", "loginUserInfo"];
            
            for(var i:int = 0 ; i < types.length ; i++) {
                var type:String = types[i];
                jsonData[type] = getSaveData(type);
            }
            
            var isValidResponse:Boolean = receiver.analyzeRefreshResponseCatchedCallByJsonData(jsonData);
        }
        
        override public function changeMap(mapImageUrl:String, 
                                  mapWidth:int,
                                  mapHeight:int, 
                                  gridColor:uint,
                                  mapMarks:Array):void {
            var mapData:Object = {
                "mapType": "imageGraphic",
                "imageSource": mapImageUrl,
                "xMax": mapWidth,
                "yMax": mapHeight,
                "gridColor": gridColor,
                "mapMarks": mapMarks};
            
            changeMapSaveData(mapData);
        }
        
        private function changeMapSaveData(mapData:Object):void {
            setSaveData('mapData', mapData);
        }
        
        override public function addCharacter(characterJsonData:Object, keyName:String = "name"):void {
            var result:Object = addCharacterData( [characterJsonData], keyName );
            printAddFailedCharacterName(result);
        }
        
        private function addCharacterData(characterDataList:Array, keyName:String):Object {
            var addFailedCharacterNames:Array = new Array;
            var characters:Array = getSaveData("characters") as Array;
            
            for(var i:int = 0 ; i < characterDataList.length ; i++) {
                var characterData:Object = characterDataList[i];
                addCharacterInOwnMap(characterData, keyName);
                
                characterData['imgId'] = createCharacterImgId();
                
                var failedName:String = isAlreadyExistCharacter(characters, characterData);
                if( failedName != null ) {
                    addFailedCharacterNames.push(failedName);
                    continue;
                }
                
                characters.push(characterData);
            }
            
            setSaveData("characters", characters);
            
            return {"addFailedCharacterNames" : addFailedCharacterNames};
        }
        
        private function getClone(obj:Object):Object {
            var buffer:ByteArray = new ByteArray();
            buffer.writeObject(obj);
            buffer.position = 0;
            var newObj:Object = buffer.readObject();
            return newObj;
        }
        
        private function addCharacterInOwnMap(characterData:Object, keyName:String):void {
            var newCharacterData:Object = getClone(characterData);
            
            newCharacterData[keyName] = "(作成中・・・)" + newCharacterData[keyName];
            receiver.addCharacterInOwnMap(newCharacterData);
        }
        
        private function isAlreadyExistCharacter(characters:Array, characterData:Object):String {
            if( characterData['name'] == null ) {
                return null;
            }
            if( characterData['name'] == "" ) {
                return null;
            }
            
            for(var i:int = 0 ; i < characters.length ; i++) { 
                var character:Object = characters[i];
                
                if( (character['imgId'] == characterData['imgId']) ||
                    (character['name'] == characterData['name']) ) {
                    
                    return characterData['name']
                        }
            }
            
            return null;
        }
        
        private var imgIdIndex:int = 0;
        
        private function createCharacterImgId(prefix:String = "character_"):String {
            imgIdIndex++;
            return (prefix + new Date().valueOf() + "_" + imgIdIndex);
        }
        
        
        override public function moveCharacter(movablePiece:MovablePiece, x:Number, y:Number):void {
            //var moveData:String = getEncodedJsonString( {"imgId": movablePiece.getId(), "x": x, "y": y});
            
            var characters:Array = getSaveData("characters") as Array;
            
            for(var i:int = 0 ; i < characters.length ; i++) {
                var characterData:Object = characters[i];
                if( characterData['imgId'] == movablePiece.getId() ) {
                    characterData['x'] = x;
                    characterData['y'] = y;
                    break;
                }
            }
            
            setSaveData("characters", characters);
        }
        
        override protected function removeCharacterByRemoveInfos(removeInfos:Array):void {
            var characters:Array = getSaveData("characters") as Array;
            var graveyard:Array = getSaveData("graveyard") as Array;
            
            for(var i:int = 0 ; i < removeInfos.length ; i++) {
                var removeCharacterData:Object = removeInfos[i];
                var removeCharacterId:String = removeCharacterData['imgId'];
                var isGotoGraveyard:Boolean = removeCharacterData['isGotoGraveyard'];
                
                removeTargetImgIdCharacter(characters, removeCharacterId, isGotoGraveyard, graveyard);
            }
            
            setSaveDataList([["characters", characters],
                             ["graveyard", graveyard]]);
        }
        
        
        public function removeTargetImgIdCharacter(characters:Array, imgId:String, isGotoGraveyard:Boolean, graveyard:Array):void {
            var index:int = -1;
            var character:Object = null;
            
            for(var i:int = 0 ; i < characters.length ; i++) {
                character = characters[i];
                if( character["imgId"] == imgId ) {
                    index = i;
                    break;
                }
            }
            
            if( index == -1 ) {
                return;
            }
            
            if( isGotoGraveyard ) {
                moveCharacterToGraveyard(character, graveyard)
            } else {
                characters.splice(index, 1); 
            }
        }
        
        private function moveCharacterToGraveyard(character:Object, graveyard:Array):void {
            graveyard.push(character);
            
            while(graveyard.length > Config.getInstance().getGraveyardLimit() ) {
                graveyard.shift();
            }
        }
        
        override public function changeCharacter(characterData:Object):void {
            var characters:Array = getSaveData("characters") as Array;
            
            for(var i:int = 0 ; i < characters.length ; i++) {
                var character:Object = characters[i];
                if(character['imgId'] == characterData['imgId']) {
                    characters[i] = characterData;
                }
            }
            
            setSaveData("characters", characters);
        }
        
        override public function sendChatMessage(chatData:Object):void {
            
            var chatMessageDataLog:Array = getSaveData('chatMessageDataLog') as Array;
            deleteOldChatMessageData(chatMessageDataLog);
            var now:Number = new Date().valueOf();
            var chatMessageData:Array = [now, chatData];
            
            chatMessageDataLog.push(chatMessageData);
            
            chatMessageDataLog.sort(function(a:Array, b:Array):Number {
                    var valueA:Number = a[0];
                    var valueB:Number = b[0];
                    
                    if( valueA > valueB ) {
                        return 1;
                    }
                    
                    if( valueA < valueB ) {
                        return -1;
                    }
                    
                    return 0;
                });
            
            setSaveData('chatMessageDataLog', chatMessageDataLog);
        }
        
        //FIXME
        private function deleteOldChatMessageData(chatMessageDataLog:Array):void {
        }
        
        override public function sendChatMessageMany(chatCharacterName:String, message:String, color:String):void {
        }
        
        override protected function sendCommandData(paramsString:String,
                                         callBack:Function = null,
                                         isRefresh:Boolean = false):void {
            Alert.show("command : " + paramsString );
        }
        
        private function sendCommandData_ToServer(paramsString:String,
                                                  callBack:Function = null,
                                                  isRefresh:Boolean = false):void {
            Log.loggingTuning("==>Begin sendCommandData_ToServer");
            Log.logging("sendCommandData_ToServer paramsString : ", paramsString);
            
            try {
                sendCommandDataCatched(paramsString,
                                       callBack,
                                       isRefresh);
            } catch( e:Error ) {
                Log.loggingException("SharedDataSender.sendCommandData_ToServer()", e);
            }
            Log.loggingTuning("==>End sendCommandData_ToServer");
        }
        
        override public function createPlayRoom(playRoomName:String,
                                       playRoomPassword:String,
                                       playRoomIndex:int,
                                       resultFunction:Function):void {
            Alert.show("createPlayRoom is NOT supported.")
        }

        override public function changePlayRoom(playRoomName:String,
                                       playRoomPassword:String,
                                       playRoomIndex:int,
                                       resultFunction:Function):void {
            Alert.show("changePlayRoom is NOT supported.");
        }
        
        override public function requestGraveyard(resultFunction:Function):void {
            var graveyard:Array = getSaveData("graveyard") as Array;
            
            var result:Array = graveyard.concat();
            result.reverse();
            
            resultFunction(result);
        }
        
        private function getUniqueIdOnWave():String {
            var viewer:WaveParticipant = wave.getViewer();
            if( viewer != null ) {
                return wave.getViewer().id;
            }
            
            return "" + (new Date().valueOf());
        }
        
        override public function requestLoginInfo(resultFunction:Function):void {
            var result:Object = {
                "playRoomStates" : [{"loginUsers":[],"passwordLockState":"--","lastUpdateTime":"---","loginUserCount":0,"index":"0","playRoomName":"samplePlayRoom"}],
                "loginMessage" : "loginMessage",
                "autoLoginRoom" : 0,
                "cardInfos" : [{"title":"TRUMP","type":"trump_jorker3"},{"title":"TORG","type":"torg"}],
                "isDiceBotOn" : true,
                "uniqueId" : getUniqueIdOnWave(),
                "refreshTimeout" : 10,
                "version" : "Ver.1.27.00(2010/07/25)",
                "playRoomMaxNumber" : 1,
                "diceBoxCgiUrl" : "customBot.pl",
                "warning" : null
            };
            
            resultFunction(result);
        }

        override protected function getUrlLoaderForSendCommand(callBack:Function):URLLoader {
            var loader:URLLoader = new URLLoader();
            
            var wrappedCallBack:Function = function(event:Event):void {
                if( callBack != null ) {
                    callBack.call(thisObj, event);
                }
                var oldLoader:URLLoader = URLLoader(event.target);
                closeLoader(oldLoader);
            }
            loader.addEventListener(Event.COMPLETE, wrappedCallBack);
            
            var wrappedCallBackForError:Function = function(event:Event):void {
                var oldLoader:URLLoader = URLLoader(event.target);
                closeLoader(oldLoader);
            }
            
            loader.addEventListener(IOErrorEvent.IO_ERROR, wrappedCallBackForError);
            loader.addEventListener(Event.UNLOAD, wrappedCallBackForError);
            
            return loader;
        }
        
        override protected function sendCommandDataCatched(paramsString:String,
                                                  callBack:Function,
                                                  isRefresh:Boolean = false):void {
            if( paramsString == null ) {
                return;
            }
            
            var request:URLRequest = getUrlRequestForSendCommandData(paramsString);
            
            if( callBack == null ) {
                Log.loggingTuning("sendToURL begin");
                sendToURL(request);
                Log.loggingTuning("sendToURL end");
                return;
            }
            
            var loader:URLLoader = null;
            
            if( isRefresh ) {
                return;
            }
            
            loader = getUrlLoaderForSendCommand(callBack);
            
            loader.load(request);
            Log.logging("loader end");
        }
        
        public static function getJsonString(jsonData:Object):String {
            return Utils.getJsonString(jsonData);
        }
        
        public static function getEncodedJsonString(jsonData:Object):String {
            var jsonString:String = getJsonString(jsonData);
            Log.logging("jsonString", jsonString);
            
            var encodeJsonString:String =  encodeURIComponent(jsonString);
            Log.logging("encodeJsonString", encodeJsonString);
            
            return encodeJsonString;
        }
        
        override public function removePlayRoom(roomNumber:int, resultFunction:Function, ignoreLoginUser:Boolean):void {
            Alert.show("removePlayRoom is NOT supported.")
        }
        
        override public function addEffect(effectData:Object):void {
            addEffectData([effectData]);
        }
        
        private function addEffectData(effectDataList:Array):void {
            var effects:Array = getSaveData('effects') as Array;
            
            for(var i:int = 0 ; i < effectDataList.length ; i++) {
                var effectData:Object = effectDataList[i];
                effectData['effectId'] = createCharacterImgId("effects_");
                effects.push(effectData);
            }
            setSaveData('effects', effects);
        }
        
        override public function changeEffect(effectData:Object):void {
            var effects:Array = getSaveData('effects') as Array;
            
            var targetId:String = effectData['effectId'];
            var index:int = -1;
            for(var i:int = 0 ; i < effects.length ; i++) {
                if( targetId == effects[i]['effectId'] ) {
                    index = i;
                    break;
                }
            }
            
            if( index == -1 ) {
                return;
            }
            
            effects[index] = effectData;
            setSaveData('effects', effects);
        }
        
        
        override public function removeEffect(targetId:String):void {
            var effects:Array = getSaveData('effects') as Array;
            
            var index:int = -1;
            for(var i:int = 0 ; i < effects.length ; i++) {
                if( targetId == effects[i]['effectId'] ) {
                    index = i;
                }
            }
            
            if( index == -1 ) {
                return;
            }
            
            effects.splice(index, 1); 
            setSaveData('effects', effects);
        }
        
    }
}
